#version 330 core
//out vec4 color;

layout (location = 0) out vec4 FragColor;
layout (location = 1) out vec4 BrightColor;

struct Material {
    sampler2D diffuse;
    sampler2D specular;
    float shininess;
};

struct DirLight {
    vec3 direction;
	
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
};

struct PointLight {
    vec3 position;
    
    float constant;
    float linear;
    float quadratic;
	
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
};

#define NR_POINT_LIGHTS 1

in VS_OUT {
    vec3 FragPos;
    vec2 TexCoords;
    vec3 TangentLightPos[NR_POINT_LIGHTS];
    vec3 TangentViewPos;
    vec3 TangentFragPos;
} fs_in;

uniform sampler2D texture_diffuse;
uniform sampler2D texture_normal;
uniform sampler2D texture_specular;
uniform samplerCube depthMap;
uniform vec3 viewPos;
uniform DirLight dirLight;
uniform PointLight pointLights[NR_POINT_LIGHTS];
uniform Material material;
uniform float far_plane;
uniform float near_plane;

vec3 gridSamplingDisk[20] = vec3[]
(
	vec3(1, 1, 1), vec3(1, -1, 1), vec3(-1, -1, 1), vec3(-1, 1, 1),
	vec3(1, 1, -1), vec3(1, -1, -1), vec3(-1, -1, -1), vec3(-1, 1, -1),
	vec3(1, 1, 0), vec3(1, -1, 0), vec3(-1, -1, 0), vec3(-1, 1, 0),
	vec3(1, 0, 1), vec3(-1, 0, 1), vec3(1, 0, -1), vec3(-1, 0, -1),
	vec3(0, 1, 1), vec3(0, -1, 1), vec3(0, -1, -1), vec3(0, 1, -1)
	);

// Function prototypes
vec3 CalcDirLight(DirLight light, vec3 normal, vec3 viewDir);
vec3 CalcPointLight(vec3 LightPos, PointLight light, vec3 normal, vec3 fragPos, vec3 viewDir, int i);

float ShadowCalculation(vec3 fragPos)
{
	float shadow = 0.0;
	float bias = 0.1;
	int samples = 20;

	vec3 fragToLight = fragPos - pointLights[0].position;
	float currentDepth = length(fragToLight);
	float viewDistance = distance(viewPos, fragPos);
	float diskRadius = (1.0 + ((viewDistance - near_plane) / (far_plane - near_plane))) / 25.0;
	
	for (int i = 0; i < samples; ++i){
		float closestDepth = texture(depthMap, fragToLight + gridSamplingDisk[i] * diskRadius).r;
		closestDepth = closestDepth * (far_plane - near_plane) + near_plane;   // Undo mapping [0;1]
		
		if (closestDepth + bias < currentDepth)
			shadow += 1.0;
	}
	shadow /= float(samples);

	return shadow;
}

void main()
{   
	// if alpha == 0 (invisible) -> discard
	vec4 texColor = texture(texture_diffuse, fs_in.TexCoords);
	if (texColor.a == 0.0)
		discard;

    // Obtain normal from normal map in range [0,1]
    vec3 normal = texture(texture_normal, fs_in.TexCoords).rgb;
    // Transform normal vector to range [-1,1]
    normal = normalize(normal * 2.0 - 1.0);
	// view direction in tangent space
	vec3 viewDir = normalize(fs_in.TangentViewPos - fs_in.TangentFragPos);

	// Phase 1: Directional lighting
	vec3 result = CalcDirLight(dirLight, normal, viewDir);
    // Phase 2: Point lights
    for(int i = 0; i < NR_POINT_LIGHTS; i++)
        result += CalcPointLight(fs_in.TangentLightPos[i], pointLights[i], normal, fs_in.TangentFragPos, viewDir, i);

	//Check whether result is higher than some threshold, if so, output as bloom threshold color
    float brightness = dot(result, vec3(0.2126, 0.7152, 0.0722));
    if(brightness > 1.0)
        BrightColor = vec4(result, texColor.a);
    else
        BrightColor = vec4(0.0, 0.0, 0.0, 1.0);
    FragColor = vec4(result, texColor.a);
}

// Calculates the color when using a directional light.
vec3 CalcDirLight(DirLight light, vec3 normal, vec3 viewDir)
{
    vec3 lightDir = normalize(-light.direction);
    // Diffuse shading
    float diff = max(dot(lightDir, normal), 0.0);
    // Specular shading
    vec3 reflectDir = reflect(-lightDir, normal);
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess);
    // Combine results
    vec3 ambient = light.ambient * vec3(texture(texture_diffuse, fs_in.TexCoords));
    vec3 diffuse = light.diffuse * diff * vec3(texture(texture_diffuse, fs_in.TexCoords));
    vec3 specular = light.specular * spec * vec3(texture(texture_specular, fs_in.TexCoords));
    
	return (ambient + diffuse + specular);
}

// Calculates the color when using a point light.
vec3 CalcPointLight(vec3 LightPos, PointLight light, vec3 normal, vec3 fragPos, vec3 viewDir, int i)
{
    vec3 lightDir = normalize(LightPos - fragPos);
    // Diffuse shading
    float diff = max(dot(lightDir, normal), 0.0);
    // Specular shading
	vec3 halfwayDir = normalize(lightDir + viewDir);  
    float spec = pow(max(dot(normal, halfwayDir), 0.0), material.shininess);
    // Attenuation
    float distance = length(LightPos - fragPos);
    float attenuation = 1.0f / (light.constant + light.linear * distance + light.quadratic * (distance * distance));    
    // Combine results
    vec3 ambient = light.ambient * vec3(texture(texture_diffuse, fs_in.TexCoords));
    vec3 diffuse = light.diffuse * diff * vec3(texture(texture_diffuse, fs_in.TexCoords));
    vec3 specular = light.specular * spec * vec3(texture(texture_specular, fs_in.TexCoords));
    ambient *= attenuation;
    diffuse *= attenuation;
    specular *= attenuation;
	
	// calc shadow only for pointlight 0
	if (i == 0) {
		float shadow = ShadowCalculation(fs_in.FragPos);
		return (ambient + (1.0 - shadow) * (diffuse + specular));
	}
	else {
		return (ambient + diffuse + specular);
	}
}